#ifndef __IPC_SDK_H__
#define __IPC_SDK_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "interface/errcode.h"

#if defined(_MSC_VER)
    #ifdef XVRSDK_STATIC
        #define XVRSDK_API     __stdcall
    #else
        #ifdef XVRSDK_EXPORTS
            #define XVRSDK_API __declspec(dllexport) __stdcall
        #else
            #define XVRSDK_API __declspec(dllimport) __stdcall
        #endif
    #endif
#else
#define XVRSDK_API
#endif

/// 注册脸图文件大小上限
#define XVR_SDK_FACE_IMAGE_SIZE_MAX         (300 * 1024)
/// 注册脸图最大像素宽度
#define XVR_SDK_FACE_IMAGE_WIDTH_MAX        1080
/// 注册脸图最大像素高度
#define XVR_SDK_FACE_IMAGE_HEIGHT_MAX       1080
/// 注册脸图特征值向量大小
#define XVR_SDK_FACE_FEATURE_VECTOR_SIZE    512
/// 注册人ID最大长度(字节)
#define XVR_SDK_PERSON_ID_MAX_LEN           33
/// 注册人名字最大长度(字节)
#define XVR_SDK_PERSON_NAME_MAX_LEN         33
/// 注册人标签最大长度(字节)
#define XVR_SDK_PERSON_LABEL_MAX_LEN        64

/// 设备时间配置
struct XVR_SDK_DevCfgTime
{
    /// 时间戳
    time_t timestamp;
    /// 时区(秒)
    int timezone;
};

/// 设备RTMP推流配置
struct XVR_SDK_DevCfgRtmp
{
    /// rtmp推流开关
    int rtmp_enable;
    /// rtmp推流超时时长
    int rtmp_timeout;
    /// rtmp 服务器地址 
    char rtmp_url[512];
};

/// 性别
typedef enum XVR_SDK_GenderType
{
    /// 未指定
    Unspecified = -1,
    /// 男
    Male = 0,
    /// 女
    Female = 1,
} XVR_SDK_GenderType;


/// 一代设备人脸注册信息
struct XVR_SDK_FaceInfoV1
{
    /// 注册人ID
    char PersonID[XVR_SDK_PERSON_ID_MAX_LEN + 1];
    /// 注册人脸ID
    char FaceID[XVR_SDK_PERSON_ID_MAX_LEN + 1];
    /// 注册人名字
    char Name[XVR_SDK_PERSON_NAME_MAX_LEN + 1];
    /// 注册人标签
    char Label[XVR_SDK_PERSON_LABEL_MAX_LEN + 1];
    /// 年龄
    int Age;
    /// 性别
    XVR_SDK_GenderType Gender;
    /// 特征值
    double Feature[XVR_SDK_FACE_FEATURE_VECTOR_SIZE];
};

/// 二代设备人脸注册信息
struct XVR_SDK_FaceInfoV2
{
    /// 注册人ID
    char PersonID[XVR_SDK_PERSON_ID_MAX_LEN + 1];
    /// 注册人名字
    char Name[XVR_SDK_PERSON_NAME_MAX_LEN + 1];
    /// 注册人标签
    char Label[XVR_SDK_PERSON_LABEL_MAX_LEN + 1];
    /// 年龄
    int Age;
    /// 性别
    XVR_SDK_GenderType Gender;
};

/// SDK连接句柄
typedef struct XSHANDLE XSHANDLE;

/// SDK配置参数
typedef enum XVR_SDK_CONFIG_TYPE
{
    /// 发送数据超时时间, 默认30秒
    XVR_SDK_CFG_SEND_TIMEOUT,
    /// 接收数据超时时间, 默认30秒
    XVR_SDK_CFG_RECV_TIMEOUT,
} XVR_SDK_CONFIG_TYPE;

/// 设备配置参数
typedef enum XVR_SDK_DEVICE_CONFIG_TYPE
{
    /// 设备时间, 见 @see struct XVR_SDK_DevCfgTime
    XVR_SDK_DEVCFG_TIME,
    /// 设备推流配置
    XVR_SDK_DEVCFG_RTMP,
} XVR_SDK_DEVICE_CONFIG_TYPE;

/**
 *  @brief SDK全局初始化
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Init(void);

/**
 *  @brief SDK全局反初始化
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Cleanup(void);

/**
 *  @brief 创建SDK连接实例
 *  @param[out] handle 连接句柄输出
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Create(XSHANDLE **handle);

/**
 *  @brief 释放并关闭SDK连接实例
 *  @param[in] handle 连接句柄
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Destroy(XSHANDLE *handle);

/**
 *  @brief 登陆设备
 *  @param[in] handle 连接句柄
 *  @param[in] address 设备地址
 *  @param[in] port 设备端口
 *  @param[in] username 用户名
 *  @param[in] password 用户密码
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Login(XSHANDLE *handle, const char *address, int port, const char *username, const char *password);

/**
 *  @brief Check connection alive
 *  @param[in] handle 连接句柄
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Heartbeat(XSHANDLE *handle);

/**
 *  @brief 获取设备配置，暂时只支持获取时间配置
 *  @param[in] handle 连接句柄
 *  @param[in] cfg 配置类型
 *  @param[out] buffer 配置数据缓冲区
 *  @param[in] size 配置数据缓冲区大小
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_GetDeviceConfig(XSHANDLE *handle, XVR_SDK_DEVICE_CONFIG_TYPE cfg, char *buffer, int size);

/**
 *  @brief 设置设备配置，暂时只支持设置时间
 *  @param[in] handle 连接句柄
 *  @param[in] cfg 配置类型
 *  @param[in] data 配置数据
 *  @param[in] length 配置数据长度
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_SetDeviceConfig(XSHANDLE *handle, XVR_SDK_DEVICE_CONFIG_TYPE cfg, const char *data, int length);

/**
 *  @brief 注册人脸(适用于二代设备)
 *  @param[in] handle 连接句柄
 *  @param[in] info 注册人脸信息
 *  @param[in] img_data 脸图数据缓冲区
 *  @param[in] img_len 脸图数据大小
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_FaceRegister(XSHANDLE *handle, const struct XVR_SDK_FaceInfoV2 *info, const char *img_data, int img_len);

/**
 *  @brief 注册人脸(适用于一代设备)
 *  @param[in] handle 连接句柄
 *  @param[in] info 注册人脸信息
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_FaceRegisterV1(XSHANDLE *handle, const struct XVR_SDK_FaceInfoV1 *info);

/**
 *  @brief 批量删除人脸
 *  @param[in] handle 连接句柄
 *  @param[in] person_ids 要删除的人脸person id数组，person id最大长度为32位。
 *  @param[in] person_cnt 要删除的人脸ID个数
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_FaceUnregister(XSHANDLE *handle, const char *person_ids[], int person_cnt);

/**
 *  @brief 查询人脸信息
 *  @param[in] handle 连接句柄
 *  @param[in] start 起始位置
 *  @param[in] limit 查询记录数, 上限50
 *  @param[in] result 查询结果(至少limit条)缓冲区
 *  @param[in] condition 查询条件, 如非NULL, 则查找PersonName或PersonID匹配condition的记录, 最大32字节
 *	@param[out] count 查询结果记录数
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_FaceList(XSHANDLE *handle, int start, int limit, struct XVR_SDK_FaceInfoV2 result[], const char *condition, int *count);

/**
 *  @brief 远程门禁控制
 *  @param[in] handle 连接句柄
 *  @param[in] gateway_index 门禁序号，从1开始，-1表示对所有门进行操作
 *  @param[in] control_type 命令值：0- 关闭，1- 打开
 *  @return 错误码
 *  @since 1.0.0
 */
int XVRSDK_API XVR_SDK_Control_Gateway(XSHANDLE *handle, int gateway_index, int control_type);

#ifdef __cplusplus
};
#endif

#endif // __IPC_SDK_H__
